-- select_type - SIMPLE
EXPLAIN SELECT * FROM tbl_user;
EXPLAIN SELECT tl.*, tu.sex,tu.remark FROM tbl_user_login_log tl LEFT JOIN tbl_user tu ON tl.user_name = tu.user_name; 

-- select_type - PRIMARY、DEPENDENT SUBQUERY
EXPLAIN SELECT * FROM tbl_user tu
WHERE EXISTS (
	SELECT * FROM tbl_user_login_log tull
	WHERE tull.user_name = tu.user_name
);

-- select_type - PRIMARY、UNION、UNION RESULT
EXPLAIN SELECT * FROM tbl_user WHERE user_name = '薛沉香'
UNION ALL
SELECT * FROM tbl_user WHERE user_name = '慕容兰娟';

-- select_type - SUBQUERY
EXPLAIN SELECT * FROM tbl_user_login_log WHERE user_name = (
	SELECT user_name FROM tbl_user WHERE id = 2
);

-- select_type - DEPENDENT SUBQUERY、DEPENDENT UNION
EXPLAIN SELECT * FROM tbl_user_login_log WHERE user_name IN(
	SELECT user_name FROM tbl_user WHERE id = 2
	UNION ALL
	SELECT user_name FROM tbl_user WHERE id = 3
);

-- 开启
SET SESSION optimizer_switch='derived_merge=on';
-- 关闭
SET SESSION optimizer_switch='derived_merge=off';
-- 会进行合并优化 select_type - DERIVED
EXPLAIN SELECT * FROM (
	SELECT * FROM tbl_user WHERE user_name = '薛沉香'
) t;

EXPLAIN SELECT * FROM tbl_user_login_log WHERE user_name IN (
	SELECT user_name FROM tbl_user WHERE id <= 3
);

-- select_type - MATERIALIZED
EXPLAIN SELECT * FROM tbl_user WHERE user_name IN (
	SELECT user_name FROM tbl_user_login_log
);

-- select_type - UNCACHEABLE SUBQUERY
EXPLAIN SELECT * FROM tbl_user WHERE user_name = ( 
	SELECT user_name FROM tbl_user_login_log WHERE id=@@sql_log_bin
);

-- select_type - UNCACHEABLE UNION
EXPLAIN SELECT * FROM tbl_user WHERE user_name IN ( 
	SELECT user_name FROM tbl_user_login_log WHERE id=@@sql_log_bin
	UNION ALL
	SELECT user_name FROM tbl_user WHERE id=@@sql_log_bin
);

-- type - system、const
SET SESSION optimizer_switch='derived_merge=off';
EXPLAIN SELECT * FROM (SELECT * FROM tbl_user WHERE id = 2) a;

-- type - eq_ref
ALTER TABLE tbl_user ADD UNIQUE (user_name);
EXPLAIN SELECT * FROM tbl_user_login_log tl 
LEFT JOIN tbl_user tu ON tl.user_name = tu.user_name;

-- type - ref
ALTER TABLE tbl_user_login_log ADD INDEX idx_user_name (user_name);
EXPLAIN SELECT * FROM tbl_user_login_log tl 
RIGHT JOIN tbl_user tu ON tl.user_name = tu.user_name;

-- type - ref_or_null
ALTER TABLE tbl_user ADD COLUMN weapon VARCHAR(50) COMMENT '武器';
UPDATE tbl_user SET weapon = '碧玉笛' WHERE user_name = '何天香';
UPDATE tbl_user SET weapon = '天问剑' WHERE user_name = '薛沉香';
UPDATE tbl_user SET weapon = '流星剑' WHERE user_name = '慕容兰娟';
UPDATE tbl_user SET weapon = '相思刀' WHERE user_name = '郭疏影';
UPDATE tbl_user SET weapon = '湘妃竹琴' WHERE user_name = '周遗梦';
UPDATE tbl_user SET weapon = '玉琵琶' WHERE user_name = '柳含姻';
ALTER TABLE tbl_user ADD INDEX idx_weapon (weapon);
EXPLAIN SELECT * FROM tbl_user WHERE weapon = '湘妃竹琴' OR weapon IS NULL;

-- type - index_merge
EXPLAIN SELECT * FROM tbl_user WHERE id = 1 OR weapon = '流星剑';

-- type - unique_subquery
EXPLAIN SELECT * FROM tbl_user t1 WHERE user_name IN (
	SELECT user_name FROM tbl_user t2 WHERE t1.weapon = t2.weapon
) OR weapon = '相思刀';

-- type - index_subquery
EXPLAIN SELECT * FROM tbl_user t1 WHERE weapon IN (
	SELECT weapon FROM tbl_user t2 WHERE t1.weapon = '玉琵琶'
) OR weapon = '相思刀';

-- type - RANGE
EXPLAIN SELECT * FROM tbl_user WHERE id < 6;

-- type - INDEX
EXPLAIN SELECT COUNT(*) FROM tbl_user;

-- type - ALL
EXPLAIN SELECT * FROM tbl_user;

-- extra - using index
EXPLAIN SELECT user_name FROM tbl_user;

-- extra - using where
EXPLAIN SELECT * FROM tbl_user WHERE sex = 0;

-- extra - using temporary
EXPLAIN SELECT sex, count(*) FROM tbl_user GROUP BY sex;

-- extra - using filesort
EXPLAIN SELECT * FROM tbl_user ORDER BY sex DESC;

-- extra - impossible where
EXPLAIN SELECT * FROM tbl_user WHERE id IS NULL;